@@ -2,7 +2,10 @@
 
 import json
 import os
+import uuid
+
 import magic
+from stix2.canonicalization.Canonicalize import canonicalize
 
 
 class ExternalReference:
@@ -38,6 +41,18 @@ class ExternalReference:
             }
         """
 
+    @staticmethod
+    def generate_id(url=None, source_name=None, external_id=None):
+        if url is not None:
+            data = {"url": url}
+        elif source_name is not None and external_id is not None:
+            data = {"source_name": source_name, "external_id": external_id}
+        else:
+            return None
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "external-reference--" + id
+
     """
         List External-Reference objects
 
