@@ -1,6 +1,9 @@
 # coding: utf-8
 
 import json
+import uuid
+
+from stix2.canonicalization.Canonicalize import canonicalize
 
 
 class IntrusionSet:
@@ -37,7 +40,7 @@ class IntrusionSet:
                                 color
                             }
                         }
-                    }                    
+                    }
                 }
                 ... on Organization {
                     x_opencti_organization_type
@@ -129,6 +132,14 @@ class IntrusionSet:
             }
         """
 
+    @staticmethod
+    def generate_id(name):
+        name = name.lower().strip()
+        data = {"name": name}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "intrusion-set--" + id
+
     """
         List Intrusion-Set objects
 
@@ -194,7 +205,7 @@ class IntrusionSet:
 
     """
         Read a Intrusion-Set object
-        
+
         :param id: the id of the Intrusion-Set
         :param filters: the filters to apply if no id provided
         :return Intrusion-Set object
@@ -326,6 +337,13 @@ class IntrusionSet:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
+
+            # Search in extensions
+            if "x_opencti_stix_ids" not in stix_object:
+                stix_object[
+                    "x_opencti_stix_ids"
+                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+
             return self.create(
                 stix_id=stix_object["id"],
                 createdBy=extras["created_by_id"]
