@@ -285,27 +285,6 @@ class StixCyberObservable:
             ... on UserAgent {
                 value
             }
-            ... on BankAccount {
-                iban
-                bic
-                account_number
-            }
-            ... on PhoneNumber {
-                value
-            }
-            ... on PaymentCard {
-                card_number
-                expiration_date
-                cvv
-                holder_name
-            }
-            ... on MediaContent {
-                title
-                content
-                media_category
-                url
-                publication_date
-            }
             importFiles {
                 edges {
                     node {
@@ -643,7 +622,7 @@ class StixCyberObservable:
             query = """
                 mutation StixCyberObservableAdd(
                     $type: String!,
-                    $stix_id: StixId,
+                    $stix_id: String,
                     $x_opencti_score: Int,
                     $x_opencti_description: String,
                     $createIndicator: Boolean,
@@ -676,10 +655,6 @@ class StixCyberObservable:
                     $Hostname: HostnameAddInput
                     $Text: TextAddInput,
                     $UserAgent: UserAgentAddInput
-                    $BankAccount: BankAccountAddInput
-                    $PhoneNumber: PhoneNumberAddInput
-                    $PaymentCard: PaymentCardAddInput
-                    $MediaContent: MediaContentAddInput
                 ) {
                     stixCyberObservableAdd(
                         type: $type,
@@ -716,10 +691,6 @@ class StixCyberObservable:
                         Hostname: $Hostname,
                         Text: $Text,
                         UserAgent: $UserAgent
-                        BankAccount: $BankAccount
-                        PhoneNumber: $PhoneNumber
-                        PaymentCard: $PaymentCard
-                        MediaContent: $MediaContent
                     ) {
                         id
                         standard_id
@@ -777,7 +748,7 @@ class StixCyberObservable:
                     "is_multipart": observable_data["is_multipart"]
                     if "is_multipart" in observable_data
                     else None,
-                    "attribute_date": observable_data["date"]
+                    "attribute_date": observable_data["attribute_date"]
                     if "date" in observable_data
                     else None,
                     "message_id": observable_data["message_id"]
@@ -823,18 +794,6 @@ class StixCyberObservable:
                     else None,
                 }
             elif type == "StixFile":
-                if (
-                    "x_opencti_additional_names" not in observable_data
-                    and self.opencti.get_attribute_in_extension(
-                        "additional_names", observable_data
-                    )
-                    is not None
-                ):
-                    observable_data[
-                        "x_opencti_additional_names"
-                    ] = self.opencti.get_attribute_in_extension(
-                        "additional_names", observable_data
-                    )
                 input_variables["StixFile"] = {
                     "hashes": hashes if len(hashes) > 0 else None,
                     "size": observable_data["size"]
@@ -1078,12 +1037,6 @@ class StixCyberObservable:
                     if "data_type" in observable_data
                     else None,
                 }
-            elif type == "User-Agent":
-                input_variables["UserAgent"] = {
-                    "value": observable_data["value"]
-                    if "value" in observable_data
-                    else None,
-                }
             elif type == "Cryptographic-Key":
                 input_variables["CryptographicKey"] = {
                     "value": observable_data["value"]
@@ -1111,51 +1064,12 @@ class StixCyberObservable:
                     if "value" in observable_data
                     else None,
                 }
-            elif type == "Bank-Account":
-                input_variables["BankAccount"] = {
-                    "iban": observable_data["iban"]
-                    if "iban" in observable_data
-                    else None,
-                    "bic": observable_data["bic"] if "bic" in observable_data else None,
-                    "account_number": observable_data["account_number"]
-                    if "account_number" in observable_data
-                    else None,
-                }
-            elif type == "Phone-Number":
-                input_variables["PhoneNumber"] = {
+            elif type == "User-Agent":
+                input_variables["UserAgent"] = {
                     "value": observable_data["value"]
                     if "value" in observable_data
                     else None,
                 }
-            elif type == "Payment-Card":
-                input_variables["PaymentCard"] = {
-                    "card_number": observable_data["card_number"]
-                    if "card_number" in observable_data
-                    else None,
-                    "expiration_date": observable_data["expiration_date"]
-                    if "expiration_date" in observable_data
-                    else None,
-                    "cvv": observable_data["cvv"] if "cvv" in observable_data else None,
-                    "holder_name": observable_data["holder_name"]
-                    if "holder_name" in observable_data
-                    else None,
-                }
-            elif type == "Media-Content":
-                input_variables["MediaContent"] = {
-                    "title": observable_data["title"]
-                    if "title" in observable_data
-                    else None,
-                    "content": observable_data["content"]
-                    if "content" in observable_data
-                    else None,
-                    "media_category": observable_data["media_category"]
-                    if "media_category" in observable_data
-                    else None,
-                    "url": observable_data["url"] if "url" in observable_data else None,
-                    "publication_date": observable_data["publication_date"]
-                    if "publication_date" in observable_data
-                    else None,
-                }
             result = self.opencti.query(query, input_variables)
             return self.opencti.process_multiple_fields(
                 result["data"]["stixCyberObservableAdd"]
@@ -1368,46 +1282,6 @@ class StixCyberObservable:
             )
             return None
 
-    """
-        Promote a Stix-Observable to an Indicator
-
-        :param id: the Stix-Observable id
-        :return void
-    """
-
-    def promote_to_indicator(self, **kwargs):
-        id = kwargs.get("id", None)
-        custom_attributes = kwargs.get("customAttributes", None)
-        if id is not None:
-            self.opencti.log("info", "Promoting Stix-Observable {" + id + "}.")
-            query = (
-                """
-                    mutation StixCyberObservableEdit($id: ID!) {
-                        stixCyberObservableEdit(id: $id) {
-                            promote {
-                                """
-                + (
-                    custom_attributes
-                    if custom_attributes is not None
-                    else self.properties
-                )
-                + """    
-                            }                               
-                        }
-                    }
-             """
-            )
-            result = self.opencti.query(query, {"id": id})
-            return self.opencti.process_multiple_fields(
-                result["data"]["stixCyberObservableEdit"]["promote"]
-            )
-        else:
-            self.opencti.log(
-                "error",
-                "[opencti_stix_cyber_observable_promote] Missing parameters: id",
-            )
-            return None
-
     """
         Delete a Stix-Observable
 
@@ -1480,7 +1354,7 @@ class StixCyberObservable:
             stix_domain_object = self.read(id=id, customAttributes=custom_attributes)
             if stix_domain_object["createdBy"] is not None:
                 query = """
-                    mutation StixCyberObservableEdit($id: ID!, $toId: StixRef! $relationship_type: String!) {
+                    mutation StixCyberObservableEdit($id: ID!, $toId: String! $relationship_type: String!) {
                         stixCyberObservableEdit(id: $id) {
                             relationDelete(toId: $toId, relationship_type: $relationship_type) {
                                 id
@@ -1613,7 +1487,7 @@ class StixCyberObservable:
                 + "}",
             )
             query = """
-               mutation StixCyberObservableRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
+               mutation StixCyberObservableRemoveRelation($id: ID!, $toId: String!, $relationship_type: String!) {
                    stixCyberObservableEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -1712,7 +1586,7 @@ class StixCyberObservable:
                 + "}",
             )
             query = """
-               mutation StixCyberObservableRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
+               mutation StixCyberObservableRemoveRelation($id: ID!, $toId: String!, $relationship_type: String!) {
                    stixCyberObservableEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -1828,7 +1702,7 @@ class StixCyberObservable:
                 + "}",
             )
             query = """
-               mutation StixCyberObservableRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
+               mutation StixCyberObservableRemoveRelation($id: ID!, $toId: String!, $relationship_type: String!) {
                    stixCyberObservableEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -1849,20 +1723,16 @@ class StixCyberObservable:
             self.opencti.log("error", "Missing parameters: id and label_id")
             return False
 
-    def push_list_export(self, file_name, data, list_filters="", mime_type=None):
+    def push_list_export(self, file_name, data, list_filters=""):
         query = """
             mutation StixCyberObservablesExportPush($file: Upload!, $listFilters: String) {
                 stixCyberObservablesExportPush(file: $file, listFilters: $listFilters)
             }
         """
-        if mime_type is None:
-            file = self.file(file_name, data)
-        else:
-            file = self.file(file_name, data, mime_type)
         self.opencti.query(
             query,
             {
-                "file": file,
+                "file": (self.file(file_name, data)),
                 "listFilters": list_filters,
             },
         )
@@ -1894,434 +1764,3 @@ class StixCyberObservable:
         )
         # return work_id
         return result["data"]["stixCoreObjectEdit"]["askEnrichment"]["id"]
-
-    """
-        Get the reports about a Stix-Cyber-Observable object
-
-        :param id: the id of the Stix-Cyber-Observable
-        :return List of reports
-    """
-
-    def reports(self, **kwargs):
-        id = kwargs.get("id", None)
-        if id is not None:
-            self.opencti.log(
-                "info",
-                "Getting reports of the Stix-Cyber-Observable {" + id + "}.",
-            )
-            query = """
-                query StixCyberObservable($id: String!) {
-                    stixCyberObservable(id: $id) {
-                        reports {
-                            edges {
-                                node {
-                                    id
-                                    standard_id
-                                    entity_type
-                                    parent_types
-                                    spec_version
-                                    created_at
-                                    updated_at
-                                    createdBy {
-                                        ... on Identity {
-                                            id
-                                            standard_id
-                                            entity_type
-                                            parent_types
-                                            spec_version
-                                            identity_class
-                                            name
-                                            description
-                                            roles
-                                            contact_information
-                                            x_opencti_aliases
-                                            created
-                                            modified
-                                            objectLabel {
-                                                edges {
-                                                    node {
-                                                        id
-                                                        value
-                                                        color
-                                                    }
-                                                }
-                                            }
-                                        }
-                                        ... on Organization {
-                                            x_opencti_organization_type
-                                            x_opencti_reliability
-                                        }
-                                        ... on Individual {
-                                            x_opencti_firstname
-                                            x_opencti_lastname
-                                        }
-                                    }
-                                    objectMarking {
-                                        edges {
-                                            node {
-                                                id
-                                                standard_id
-                                                entity_type
-                                                definition_type
-                                                definition
-                                                created
-                                                modified
-                                                x_opencti_order
-                                                x_opencti_color
-                                            }
-                                        }
-                                    }
-                                    objectLabel {
-                                        edges {
-                                            node {
-                                                id
-                                                value
-                                                color
-                                            }
-                                        }
-                                    }
-                                    externalReferences {
-                                        edges {
-                                            node {
-                                                id
-                                                standard_id
-                                                entity_type
-                                                source_name
-                                                description
-                                                url
-                                                hash
-                                                external_id
-                                                created
-                                                modified
-                                                importFiles {
-                                                    edges {
-                                                        node {
-                                                            id
-                                                            name
-                                                            size
-                                                            metaData {
-                                                                mimetype
-                                                                version
-                                                            }
-                                                        }
-                                                    }
-                                                }
-                                            }
-                                        }
-                                    }
-                                    revoked
-                                    confidence
-                                    created
-                                    modified
-                                    name
-                                    description
-                                    report_types
-                                    published                                    
-                                }
-                            }
-                        }
-                    }
-                }
-             """
-            result = self.opencti.query(query, {"id": id})
-            processed_result = self.opencti.process_multiple_fields(
-                result["data"]["stixCyberObservable"]
-            )
-            if processed_result:
-                return processed_result["reports"]
-            else:
-                return []
-        else:
-            self.opencti.log("error", "Missing parameters: id")
-            return None
-
-    """
-        Get the notes about a Stix-Cyber-Observable object
-
-        :param id: the id of the Stix-Cyber-Observable
-        :return List of notes
-    """
-
-    def notes(self, **kwargs):
-        id = kwargs.get("id", None)
-        if id is not None:
-            self.opencti.log(
-                "info",
-                "Getting notes of the Stix-Cyber-Observable {" + id + "}.",
-            )
-            query = """
-                query StixCyberObservable($id: String!) {
-                    stixCyberObservable(id: $id) {
-                        notes {
-                            edges {
-                                node {
-                                    id
-                                    standard_id
-                                    entity_type
-                                    parent_types
-                                    spec_version
-                                    created_at
-                                    updated_at
-                                    createdBy {
-                                        ... on Identity {
-                                            id
-                                            standard_id
-                                            entity_type
-                                            parent_types
-                                            spec_version
-                                            identity_class
-                                            name
-                                            description
-                                            roles
-                                            contact_information
-                                            x_opencti_aliases
-                                            created
-                                            modified
-                                            objectLabel {
-                                                edges {
-                                                    node {
-                                                        id
-                                                        value
-                                                        color
-                                                    }
-                                                }
-                                            }
-                                        }
-                                        ... on Organization {
-                                            x_opencti_organization_type
-                                            x_opencti_reliability
-                                        }
-                                        ... on Individual {
-                                            x_opencti_firstname
-                                            x_opencti_lastname
-                                        }
-                                    }
-                                    objectMarking {
-                                        edges {
-                                            node {
-                                                id
-                                                standard_id
-                                                entity_type
-                                                definition_type
-                                                definition
-                                                created
-                                                modified
-                                                x_opencti_order
-                                                x_opencti_color
-                                            }
-                                        }
-                                    }
-                                    objectLabel {
-                                        edges {
-                                            node {
-                                                id
-                                                value
-                                                color
-                                            }
-                                        }
-                                    }
-                                    externalReferences {
-                                        edges {
-                                            node {
-                                                id
-                                                standard_id
-                                                entity_type
-                                                source_name
-                                                description
-                                                url
-                                                hash
-                                                external_id
-                                                created
-                                                modified
-                                                importFiles {
-                                                    edges {
-                                                        node {
-                                                            id
-                                                            name
-                                                            size
-                                                            metaData {
-                                                                mimetype
-                                                                version
-                                                            }
-                                                        }
-                                                    }
-                                                }
-                                            }
-                                        }
-                                    }
-                                    revoked
-                                    confidence
-                                    created
-                                    modified
-                                    attribute_abstract
-                                    content
-                                    authors                                
-                                }
-                            }
-                        }
-                    }
-                }
-             """
-            result = self.opencti.query(query, {"id": id})
-            processed_result = self.opencti.process_multiple_fields(
-                result["data"]["stixCyberObservable"]
-            )
-            if processed_result:
-                return processed_result["notes"]
-            else:
-                return []
-        else:
-            self.opencti.log("error", "Missing parameters: id")
-            return None
-
-    """
-        Get the observed data of a Stix-Cyber-Observable object
-
-        :param id: the id of the Stix-Cyber-Observable
-        :return List of observed data
-    """
-
-    def observed_data(self, **kwargs):
-        id = kwargs.get("id", None)
-        if id is not None:
-            self.opencti.log(
-                "info",
-                "Getting Observed-Data of the Stix-Cyber-Observable {" + id + "}.",
-            )
-            query = """
-                    query StixCyberObservable($id: String!) {
-                        stixCyberObservable(id: $id) {
-                            observedData {
-                                edges {
-                                    node {
-                                        id
-                                        standard_id
-                                        entity_type
-                                        parent_types
-                                        spec_version
-                                        created_at
-                                        updated_at
-                                        createdBy {
-                                            ... on Identity {
-                                                id
-                                                standard_id
-                                                entity_type
-                                                parent_types
-                                                spec_version
-                                                identity_class
-                                                name
-                                                description
-                                                roles
-                                                contact_information
-                                                x_opencti_aliases
-                                                created
-                                                modified
-                                                objectLabel {
-                                                    edges {
-                                                        node {
-                                                            id
-                                                            value
-                                                            color
-                                                        }
-                                                    }
-                                                }
-                                            }
-                                            ... on Organization {
-                                                x_opencti_organization_type
-                                                x_opencti_reliability
-                                            }
-                                            ... on Individual {
-                                                x_opencti_firstname
-                                                x_opencti_lastname
-                                            }
-                                        }
-                                        objectMarking {
-                                            edges {
-                                                node {
-                                                    id
-                                                    standard_id
-                                                    entity_type
-                                                    definition_type
-                                                    definition
-                                                    created
-                                                    modified
-                                                    x_opencti_order
-                                                    x_opencti_color
-                                                }
-                                            }
-                                        }
-                                        objectLabel {
-                                            edges {
-                                                node {
-                                                    id
-                                                    value
-                                                    color
-                                                }
-                                            }
-                                        }
-                                        externalReferences {
-                                            edges {
-                                                node {
-                                                    id
-                                                    standard_id
-                                                    entity_type
-                                                    source_name
-                                                    description
-                                                    url
-                                                    hash
-                                                    external_id
-                                                    created
-                                                    modified
-                                                    importFiles {
-                                                        edges {
-                                                            node {
-                                                                id
-                                                                name
-                                                                size
-                                                                metaData {
-                                                                    mimetype
-                                                                    version
-                                                                }
-                                                            }
-                                                        }
-                                                    }
-                                                }
-                                            }
-                                        }
-                                        revoked
-                                        confidence
-                                        created
-                                        modified
-                                        first_observed
-                                        last_observed
-                                        number_observed
-                                        importFiles {
-                                            edges {
-                                                node {
-                                                    id
-                                                    name
-                                                    size
-                                                    metaData {
-                                                        mimetype
-                                                        version
-                                                    }
-                                                }
-                                            }
-                                        }    
-                                    }
-                                }
-                            }
-                        }
-                    }
-                 """
-            result = self.opencti.query(query, {"id": id})
-            processed_result = self.opencti.process_multiple_fields(
-                result["data"]["stixCyberObservable"]
-            )
-            if processed_result:
-                return processed_result["observedData"]
-            else:
-                return []
-        else:
-            self.opencti.log("error", "Missing parameters: id")
-            return None
