@@ -25,6 +25,7 @@ class StixDomainObject:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -40,7 +41,7 @@ class StixDomainObject:
                                 color
                             }
                         }
-                    }                    
+                    }
                 }
                 ... on Organization {
                     x_opencti_organization_type
@@ -88,6 +89,19 @@ class StixDomainObject:
                         external_id
                         created
                         modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
@@ -107,7 +121,7 @@ class StixDomainObject:
                     edges {
                         node {
                             id
-                            standard_id                            
+                            standard_id
                             entity_type
                             kill_chain_name
                             phase_name
@@ -135,13 +149,17 @@ class StixDomainObject:
                         node {
                             ... on BasicObject {
                                 id
+                                entity_type
+                                standard_id
                             }
                             ... on BasicRelationship {
                                 id
-                            } 
+                                entity_type
+                                standard_id
+                            }
                         }
                     }
-                }                
+                }
             }
             ... on ObservedData {
                 first_observed
@@ -152,13 +170,17 @@ class StixDomainObject:
                         node {
                             ... on BasicObject {
                                 id
+                                entity_type
+                                standard_id
                             }
                             ... on BasicRelationship {
                                 id
-                            } 
+                                entity_type
+                                standard_id
+                            }
                         }
                     }
-                }                
+                }
             }
             ... on Opinion {
                 explanation
@@ -169,29 +191,36 @@ class StixDomainObject:
                         node {
                             ... on BasicObject {
                                 id
+                                entity_type
+                                standard_id
                             }
                             ... on BasicRelationship {
                                 id
-                            } 
+                                entity_type
+                                standard_id
+                            }
                         }
                     }
-                }                
+                }
             }
             ... on Report {
                 name
                 description
                 report_types
                 published
-                x_opencti_report_status
                 objects {
                     edges {
                         node {
                             ... on BasicObject {
                                 id
+                                entity_type
+                                standard_id
                             }
                             ... on BasicRelationship {
                                 id
-                            } 
+                                entity_type
+                                standard_id
+                            }
                         }
                     }
                 }
@@ -223,6 +252,11 @@ class StixDomainObject:
                 x_opencti_aliases
                 contact_information
             }
+            ... on System {
+                name
+                description
+                x_opencti_aliases
+            }
             ... on Indicator {
                 pattern_type
                 pattern_version
@@ -304,7 +338,7 @@ class StixDomainObject:
                     edges {
                         node {
                             id
-                            standard_id                            
+                            standard_id
                             entity_type
                             kill_chain_name
                             phase_name
@@ -374,6 +408,10 @@ class StixDomainObject:
                         id
                         name
                         size
+                        metaData {
+                            mimetype
+                            version
+                        }
                     }
                 }
             }
@@ -474,7 +512,7 @@ class StixDomainObject:
 
     """
         Read a Stix-Domain-Object object
-        
+
         :param id: the id of the Stix-Domain-Object
         :param types: list of Stix Domain Entity types
         :param filters: the filters to apply if no id provided
@@ -567,28 +605,18 @@ class StixDomainObject:
         Update a Stix-Domain-Object object field
 
         :param id: the Stix-Domain-Object id
-        :param key: the key of the field
-        :param value: the value of the field
-        :return The updated Stix-Domain-Object object
+        :param input: the input of the field
     """
 
     def update_field(self, **kwargs):
         id = kwargs.get("id", None)
-        key = kwargs.get("key", None)
-        value = kwargs.get("value", None)
-        operation = kwargs.get("operation", "replace")
-        if isinstance(value, list):
-            value = [str(v) for v in value]
-        else:
-            value = str(value)
-        if id is not None and key is not None and value is not None:
-            self.opencti.log(
-                "info", "Updating Stix-Domain-Object {" + id + "} field {" + key + "}."
-            )
+        input = kwargs.get("input", None)
+        if id is not None and input is not None:
+            self.opencti.log("info", "Updating Stix-Domain-Object {" + id + "}")
             query = """
-                    mutation StixDomainObjectEdit($id: ID!, $input: EditInput!, $operation: EditOperation) {
+                    mutation StixDomainObjectEdit($id: ID!, $input: [EditInput]!) {
                         stixDomainObjectEdit(id: $id) {
-                            fieldPatch(input: $input, operation: $operation) {
+                            fieldPatch(input: $input) {
                                 id
                                 standard_id
                                 entity_type
@@ -600,8 +628,7 @@ class StixDomainObject:
                 query,
                 {
                     "id": id,
-                    "input": {"key": key, "value": value},
-                    "operation": operation,
+                    "input": input,
                 },
             )
             return self.opencti.process_multiple_fields(
@@ -610,7 +637,7 @@ class StixDomainObject:
         else:
             self.opencti.log(
                 "error",
-                "[opencti_stix_domain_object] Missing parameters: id and key and value",
+                "[opencti_stix_domain_object] Missing parameters: id and input",
             )
             return None
 
@@ -640,7 +667,7 @@ class StixDomainObject:
             return None
 
     """
-        Upload a file in this Stix-Domain-Object 
+        Upload a file in this Stix-Domain-Object
 
         :param id: the Stix-Domain-Object id
         :param file_name
@@ -654,41 +681,35 @@ class StixDomainObject:
         data = kwargs.get("data", None)
         mime_type = kwargs.get("mime_type", "text/plain")
         if id is not None and file_name is not None:
-            stix_domain_object = self.read(id=id)
-            if stix_domain_object is None:
-                self.opencti.log("error", "Cannot add File, entity not found")
-                return False
             final_file_name = os.path.basename(file_name)
-            current_files = {}
-            for file in stix_domain_object["importFiles"]:
-                current_files[file["name"]] = file
-            if final_file_name in current_files:
-                return current_files[final_file_name]
-            else:
-                self.opencti.log(
-                    "info", "Uploading a file in Stix-Domain-Object {" + id + "}."
-                )
-                query = """
-                    mutation StixDomainObjectEdit($id: ID!, $file: Upload!) {
-                        stixDomainObjectEdit(id: $id) {
-                            importPush(file: $file) {
-                                id
-                                name
-                            }
+            query = """
+                mutation StixDomainObjectEdit($id: ID!, $file: Upload!) {
+                    stixDomainObjectEdit(id: $id) {
+                        importPush(file: $file) {
+                            id
+                            name
                         }
                     }
-                 """
-                if data is None:
-                    data = open(file_name, "rb")
-                    if file_name.endswith(".json"):
-                        mime_type = "application/json"
-                    else:
-                        mime_type = magic.from_file(file_name, mime=True)
-
-                return self.opencti.query(
-                    query,
-                    {"id": id, "file": (self.file(final_file_name, data, mime_type))},
-                )
+                }
+             """
+            if data is None:
+                data = open(file_name, "rb")
+                if file_name.endswith(".json"):
+                    mime_type = "application/json"
+                else:
+                    mime_type = magic.from_file(file_name, mime=True)
+            self.opencti.log(
+                "info",
+                "Uploading a file {"
+                + final_file_name
+                + "} in Stix-Domain-Object {"
+                + id
+                + "}.",
+            )
+            return self.opencti.query(
+                query,
+                {"id": id, "file": (self.file(final_file_name, data, mime_type))},
+            )
         else:
             self.opencti.log(
                 "error",
@@ -700,7 +721,7 @@ class StixDomainObject:
         query = """
             mutation StixDomainObjectsExportPush($type: String!, $file: Upload!, $listFilters: String) {
                 stixDomainObjectsExportPush(type: $type, file: $file, listFilters: $listFilters)
-            } 
+            }
         """
         self.opencti.query(
             query,
@@ -717,7 +738,7 @@ class StixDomainObject:
                 stixDomainObjectEdit(id: $id) {
                     exportPush(file: $file)
                 }
-            } 
+            }
         """
         self.opencti.query(
             query, {"id": entity_id, "file": (self.file(file_name, data))}
@@ -1220,13 +1241,12 @@ class StixDomainObject:
                                     standard_id
                                     entity_type
                                     spec_version
-                                    
+
                                     name
                                     alias
                                     description
                                     report_class
                                     published
-                                    object_status
                                     source_confidence_level
                                     graph_data
                                     created
@@ -1279,7 +1299,7 @@ class StixDomainObject:
                                                 id
                                             }
                                         }
-                                    }            
+                                    }
                                     externalReferences {
                                         edges {
                                             node {
@@ -1435,7 +1455,7 @@ class StixDomainObject:
                                                 id
                                             }
                                         }
-                                    }            
+                                    }
                                     externalReferences {
                                         edges {
                                             node {
