@@ -1,5 +1,10 @@
 # coding: utf-8
 
+import datetime
+import uuid
+
+from stix2.canonicalization.Canonicalize import canonicalize
+
 
 class StixSightingRelationship:
     def __init__(self, opencti):
@@ -26,6 +31,7 @@ class StixSightingRelationship:
                     standard_id
                     entity_type
                     parent_types
+                    identity_class
                     name
                     description
                     roles
@@ -41,7 +47,7 @@ class StixSightingRelationship:
                                 color
                             }
                         }
-                    }                    
+                    }
                 }
                 ... on Organization {
                     x_opencti_organization_type
@@ -126,6 +132,9 @@ class StixSightingRelationship:
                 }
                 ... on Sector {
                     name
+                }
+                 ... on System {
+                    name
                 }
                 ... on Indicator {
                     name
@@ -162,10 +171,10 @@ class StixSightingRelationship:
                 }
                 ... on Incident {
                     name
-                }           
+                }
                 ... on StixCyberObservable {
                     observable_value
-                }                     
+                }
                 ... on StixCoreRelationship {
                     standard_id
                     spec_version
@@ -208,6 +217,9 @@ class StixSightingRelationship:
                 ... on Sector {
                     name
                 }
+                ... on System {
+                    name
+                }
                 ... on Indicator {
                     name
                 }
@@ -256,6 +268,29 @@ class StixSightingRelationship:
             }
         """
 
+    @staticmethod
+    def generate_id(source_ref, target_ref, first_seen=None, last_seen=None):
+        if isinstance(first_seen, datetime.datetime):
+            first_seen = first_seen.isoformat()
+        if isinstance(last_seen, datetime.datetime):
+            last_seen = last_seen.isoformat()
+
+        if first_seen is not None and last_seen is not None:
+            data = {
+                "source_ref": source_ref,
+                "target_ref": target_ref,
+                "first_seen": first_seen,
+                "last_seen": last_seen,
+            }
+        else:
+            data = {
+                "source_ref": source_ref,
+                "target_ref": target_ref,
+            }
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "sighting--" + id
+
     """
         List stix_sightings objects
 
@@ -285,6 +320,7 @@ class StixSightingRelationship:
         after = kwargs.get("after", None)
         order_by = kwargs.get("orderBy", None)
         order_mode = kwargs.get("orderMode", None)
+        custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
         if get_all:
@@ -305,7 +341,7 @@ class StixSightingRelationship:
                         edges {
                             node {
                                 """
-            + self.properties
+            + (custom_attributes if custom_attributes is not None else self.properties)
             + """
                         }
                     }
@@ -468,6 +504,7 @@ class StixSightingRelationship:
         object_marking = kwargs.get("objectMarking", None)
         object_label = kwargs.get("objectLabel", None)
         external_references = kwargs.get("externalReferences", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         update = kwargs.get("update", False)
 
         self.opencti.log(
@@ -503,6 +540,7 @@ class StixSightingRelationship:
                     "objectMarking": object_marking,
                     "objectLabel": object_label,
                     "externalReferences": external_references,
+                    "x_opencti_stix_ids": x_opencti_stix_ids,
                     "update": update,
                 }
             },
@@ -515,21 +553,17 @@ class StixSightingRelationship:
         Update a stix_sighting object field
 
         :param id: the stix_sighting id
-        :param key: the key of the field
-        :param value: the value of the field
+        :param input: the input of the field
         :return The updated stix_sighting object
     """
 
     def update_field(self, **kwargs):
         id = kwargs.get("id", None)
-        key = kwargs.get("key", None)
-        value = kwargs.get("value", None)
-        if id is not None and key is not None and value is not None:
-            self.opencti.log(
-                "info", "Updating stix_sighting {" + id + "} field {" + key + "}."
-            )
+        input = kwargs.get("input", None)
+        if id is not None and input is not None:
+            self.opencti.log("info", "Updating stix_sighting {" + id + "}")
             query = """
-                    mutation StixSightingRelationshipEdit($id: ID!, $input: EditInput!) {
+                    mutation StixSightingRelationshipEdit($id: ID!, $input: [EditInput]!) {
                         stixSightingRelationshipEdit(id: $id) {
                             fieldPatch(input: $input) {
                                 id
@@ -538,7 +572,11 @@ class StixSightingRelationship:
                     }
                 """
             result = self.opencti.query(
-                query, {"id": id, "input": {"key": key, "value": value}}
+                query,
+                {
+                    "id": id,
+                    "input": input,
+                },
             )
             return self.opencti.process_multiple_fields(
                 result["data"]["stixSightingRelationshipEdit"]["fieldPatch"]
@@ -550,6 +588,209 @@ class StixSightingRelationship:
             )
             return None
 
+    """
+        Add a Marking-Definition object to stix_sighting_relationship object (object_marking_refs)
+
+        :param id: the id of the stix_sighting_relationship
+        :param marking_definition_id: the id of the Marking-Definition
+        :return Boolean
+    """
+
+    def add_marking_definition(self, **kwargs):
+        id = kwargs.get("id", None)
+        marking_definition_id = kwargs.get("marking_definition_id", None)
+        if id is not None and marking_definition_id is not None:
+            custom_attributes = """
+                id
+                objectMarking {
+                    edges {
+                        node {
+                            id
+                            standard_id
+                            entity_type
+                            definition_type
+                            definition
+                            x_opencti_order
+                            x_opencti_color
+                            created
+                            modified
+                        }
+                    }
+                }
+            """
+            stix_core_relationship = self.read(
+                id=id, customAttributes=custom_attributes
+            )
+            if stix_core_relationship is None:
+                self.opencti.log(
+                    "error", "Cannot add Marking-Definition, entity not found"
+                )
+                return False
+            if marking_definition_id in stix_core_relationship["objectMarkingIds"]:
+                return True
+            else:
+                self.opencti.log(
+                    "info",
+                    "Adding Marking-Definition {"
+                    + marking_definition_id
+                    + "} to stix_sighting_relationship {"
+                    + id
+                    + "}",
+                )
+                query = """
+                   mutation StixSightingRelationshipEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
+                       stixSightingRelationshipEdit(id: $id) {
+                            relationAdd(input: $input) {
+                                id
+                            }
+                       }
+                   }
+                """
+                self.opencti.query(
+                    query,
+                    {
+                        "id": id,
+                        "input": {
+                            "toId": marking_definition_id,
+                            "relationship_type": "object-marking",
+                        },
+                    },
+                )
+                return True
+        else:
+            self.opencti.log(
+                "error", "Missing parameters: id and marking_definition_id"
+            )
+            return False
+
+    """
+        Remove a Marking-Definition object to stix_sighting_relationship
+
+        :param id: the id of the stix_sighting_relationship
+        :param marking_definition_id: the id of the Marking-Definition
+        :return Boolean
+    """
+
+    def remove_marking_definition(self, **kwargs):
+        id = kwargs.get("id", None)
+        marking_definition_id = kwargs.get("marking_definition_id", None)
+        if id is not None and marking_definition_id is not None:
+            self.opencti.log(
+                "info",
+                "Removing Marking-Definition {"
+                + marking_definition_id
+                + "} from stix_sighting_relationship {"
+                + id
+                + "}",
+            )
+            query = """
+               mutation StixSightingRelationshipEdit($id: ID!, $toId: String!, $relationship_type: String!) {
+                   stixSightingRelationshipEdit(id: $id) {
+                        relationDelete(toId: $toId, relationship_type: $relationship_type) {
+                            id
+                        }
+                   }
+               }
+            """
+            self.opencti.query(
+                query,
+                {
+                    "id": id,
+                    "toId": marking_definition_id,
+                    "relationship_type": "object-marking",
+                },
+            )
+            return True
+        else:
+            self.opencti.log("error", "Missing parameters: id and label_id")
+            return False
+
+    """
+        Update the Identity author of a stix_sighting_relationship object (created_by)
+
+        :param id: the id of the stix_sighting_relationship
+        :param identity_id: the id of the Identity
+        :return Boolean
+    """
+
+    def update_created_by(self, **kwargs):
+        id = kwargs.get("id", None)
+        identity_id = kwargs.get("identity_id", None)
+        if id is not None:
+            self.opencti.log(
+                "info",
+                "Updating author of stix_sighting_relationship {"
+                + id
+                + "} with Identity {"
+                + str(identity_id)
+                + "}",
+            )
+            custom_attributes = """
+                id
+                createdBy {
+                    ... on Identity {
+                        id
+                        standard_id
+                        entity_type
+                        parent_types
+                        name
+                        x_opencti_aliases
+                        description
+                        created
+                        modified
+                    }
+                    ... on Organization {
+                        x_opencti_organization_type
+                        x_opencti_reliability
+                    }
+                    ... on Individual {
+                        x_opencti_firstname
+                        x_opencti_lastname
+                    }
+                }
+            """
+            stix_domain_object = self.read(id=id, customAttributes=custom_attributes)
+            if stix_domain_object["createdBy"] is not None:
+                query = """
+                    mutation StixSightingRelationshipEdit($id: ID!, $toId: String! $relationship_type: String!) {
+                        stixSightingRelationshipEdit(id: $id) {
+                            relationDelete(toId: $toId, relationship_type: $relationship_type) {
+                                id
+                            }
+                        }
+                    }
+                """
+                self.opencti.query(
+                    query,
+                    {
+                        "id": id,
+                        "toId": stix_domain_object["createdBy"]["id"],
+                        "relationship_type": "created-by",
+                    },
+                )
+            if identity_id is not None:
+                # Add the new relation
+                query = """
+                    mutation StixSightingRelationshipEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
+                        stixSightingRelationshipEdit(id: $id) {
+                            relationAdd(input: $input) {
+                                id
+                            }
+                        }
+                    }
+               """
+                variables = {
+                    "id": id,
+                    "input": {
+                        "toId": identity_id,
+                        "relationship_type": "created-by",
+                    },
+                }
+                self.opencti.query(query, variables)
+        else:
+            self.opencti.log("error", "Missing parameters: id")
+            return False
+
     """
         Delete a stix_sighting
 
