@@ -1,8 +1,11 @@
 # coding: utf-8
 
 import json
+import uuid
 from typing import Union
 
+from stix2.canonicalization.Canonicalize import canonicalize
+
 
 class ThreatActor:
     """Main ThreatActor class for OpenCTI
@@ -29,6 +32,7 @@ class ThreatActor:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -44,7 +48,7 @@ class ThreatActor:
                                 color
                             }
                         }
-                    }                    
+                    }
                 }
                 ... on Organization {
                     x_opencti_organization_type
@@ -92,6 +96,19 @@ class ThreatActor:
                         external_id
                         created
                         modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
@@ -110,10 +127,31 @@ class ThreatActor:
             sophistication
             resource_level
             primary_motivation
-            secondary_motivations      
+            secondary_motivations
             personal_motivations
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
         """
 
+    @staticmethod
+    def generate_id(name):
+        name = name.lower().strip()
+        data = {"name": name}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "threat-actor--" + id
+
     def list(self, **kwargs) -> dict:
         """List Threat-Actor objects
 
@@ -136,6 +174,7 @@ class ThreatActor:
         after = kwargs.get("after", None)
         order_by = kwargs.get("orderBy", None)
         order_mode = kwargs.get("orderMode", None)
+        custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
         if get_all:
@@ -151,7 +190,7 @@ class ThreatActor:
                     edges {
                         node {
                             """
-            + self.properties
+            + (custom_attributes if custom_attributes is not None else self.properties)
             + """
                         }
                     }
@@ -288,6 +327,7 @@ class ThreatActor:
         primary_motivation = kwargs.get("primary_motivation", None)
         secondary_motivations = kwargs.get("secondary_motivations", None)
         personal_motivations = kwargs.get("personal_motivations", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         update = kwargs.get("update", False)
 
         if name is not None and description is not None:
@@ -328,6 +368,7 @@ class ThreatActor:
                         "primary_motivation": primary_motivation,
                         "secondary_motivations": secondary_motivations,
                         "personal_motivations": personal_motivations,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
                         "update": update,
                     }
                 },
@@ -353,6 +394,13 @@ class ThreatActor:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
+
+            # Search in extensions
+            if "x_opencti_stix_ids" not in stix_object:
+                stix_object[
+                    "x_opencti_stix_ids"
+                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+
             return self.create(
                 stix_id=stix_object["id"],
                 createdBy=extras["created_by_id"]
@@ -406,6 +454,9 @@ class ThreatActor:
                 personal_motivations=stix_object["personal_motivations"]
                 if "personal_motivations" in stix_object
                 else None,
+                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
+                if "x_opencti_stix_ids" in stix_object
+                else None,
                 update=update,
             )
         else:
